<?php
/*
*	Plugin Name: Parental Control
*    	Plugin URI: http://mummila.net/nuudelisoppa/index.php/?page_id=1352
*   	Description: Filters profanity from posts and comments, excludes adult-rated posts from front page and single post view based on user choice (a cookie).
* 	Author: Jani Uusitalo
* 	Author URI: http://mummila.net/nuudelisoppa
* 	Version: 1.0.23 Beta
*
* 
*	This program is free software; you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation; either version 2 of the License, or
*	(at your option) any later version.
*	
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
*	Version History
*
*		(2006-08-26)
*		0.9.5:	Configuration implemented.
*		0.9.8:	Profanity filter re-written from scratch to retain capitalization of word beginnings and ends, automatically replace only whole words.
*		0.9.8.2:	Profanity filter configuration now rejects "words" that are too short (less than two characters).
*		0.9.9:	Profanity filter improved:
*				-	profanities at the beginning of a comment weren't censored, now they are
*				-	a huge chunk of regexps condensed into two simple ones.
*		1.0 Beta:	First release. 
*		1.0.23 Beta:	Wordpress 2.3 compatibility.
*/

/*
* TO DO (in order of priority, from highest to lowest):
*	-	improve profanity filter to allow words to be selectable as being matched only as whole words. example: "ass" currently matches "assert", which is undesirable.
*	-	provide customized feeds with filtering on?
*
*/



function jupc_switch() {
	$home = get_settings('home');
	$toggler = $home."/?kiddie_protect=toggle";
	if (jupc_on())
		echo '<a id="jupcswitch" href="'.$toggler.'" title="Turn off Parental Control">Turn off Parental Control</a>';
	else
		echo '<a id="jupcswitch" href="'.$toggler.'" title="Turn on Parental Control">Turn on Parental Control</a>';
}

function jupc_excat_set() {
	global $jupc_excluded_cat;
	if ($jupc_excluded_cat < '0')
		return false;
	else
		return true;
}

function jupc_is_excluded_post() {
	global $post;
	global $jupc_excluded_cat;
	if (jupc_excat_set())
		return in_category($jupc_excluded_cat);
	else
		return false;
}

function jupc_exclusion($content) {
	if (jupc_on() && jupc_is_excluded_post() && !is_feed() && !is_admin())
		return "<p>This post has been suppressed due to Parental Control being in effect.</p>";
	else
		return $content;
}

// "Because setting a cookie with a value of FALSE will try to delete the cookie, you should not use boolean values. Instead, use 0 for FALSE  and 1 for TRUE." (http://fi2.php.net/setcookie)
function jupc_on() {
	if (!isset($_COOKIE['jupg_on']))
		return true;
	else if ($_COOKIE['jupg_on'] == "1")
		return true;
	else
		return false;
}

// adapted from Smart Unread Comments
function jupc_toggle() {
	if (jupc_on())
		$jupc_new = "0";
	else
		$jupc_new = "1";

	$timeout = 3600*24*90;	// 90 days
	$time = strtotime(gmdate('Y-m-d H:i:s')) + get_settings('gmt_offset') * 3600;
	$cookiepath = parse_url(get_bloginfo('siteurl'));
	if ($cookiepath['path'] == "") {
		$cookiepath['path'] = "/";
	}
	
	setcookie("jupg_on", $jupc_new, $time+$timeout, $cookiepath['path']);

}

function jupc_profanity_filter($content) {
	global $jupc_profanities;
	jupc_load_profanities();

	// set the replacing character
	$censoredchar = '*';
	
	if (jupc_on() && !is_feed() && !is_admin() && !empty($jupc_profanities)) {
		// foreach nasty word in database...
		foreach($jupc_profanities as $nastyword) {

			// test for sanity
			if (strlen($nastyword) > 1) {

				// using the i flag in the preg_replace below won't work, because the substitute word needs to retain capitalization.
				// therefore, the cases where words begin and/or end in capitalized letters are dealt with alongside all lowercaps cases.
				// because everything between the first and the last letter is censored, there's no need to handle all variants, four's enough
				$Nastyword	=	strtoupper(substr($nastyword, 0, 1));			// first
				$Nastyword	.=	substr($nastyword, 1, strlen($nastyword)-2);	// middle
				$Nastyword	.=	substr($nastyword, -1);							//last
				$NastyworD	=	strtoupper(substr($nastyword, 0, 1));			// first
				$NastyworD	.=	substr($nastyword, 1, strlen($nastyword)-2);	// middle
				$NastyworD	.=	strtoupper(substr($nastyword, -1));				//last
				$nastyworD	=	substr($nastyword, 0, 1);						// first
				$nastyworD	.=	substr($nastyword, 1, strlen($nastyword)-2);	// middle
				$nastyworD	.=	strtoupper(substr($nastyword, -1));				//last
				
				$nasties = array ($nastyword, $Nastyword, $NastyworD, $nastyworD);
				foreach($nasties as $nasty) {
					$nastymatch = array (
					'/(\W)'	. $nasty . '/',
					'/^' . $nasty . '/'
					);

					// make up the censored word by replacing all between the first and the last character with $censoredchar.
					$nastyfirst = $censoredchar;	// censor two-letter $nastywords in their entirety.
					$nastylast  = $censoredchar;	//
					if (strlen($nasty) > 2) {		//
						$nastyfirst = substr($nasty, 0, 1);
						$nastylast  = substr($nasty, -1);
					}
					$censoredword = $nastyfirst;
					for ($i = 1; $i < (strlen($nastyword)-1); $i++) {
						$censoredword .= $censoredchar;
					}
					$censoredword .= $nastylast;
					$censored = array (
						'$1' . $censoredword,
						$censoredword
						);


					// find all instances of current nasty word, replace with censored version
					$content = preg_replace($nastymatch, $censored, $content);
				}
			} // sanity test
		} // foreach nasty word in database
	}
	return $content;
}

if (isset($_REQUEST["kiddie_protect"])) {
	jupc_toggle();
	if ( isset($_SERVER['HTTP_REFERER']) && $_SERVER['HTTP_REFERER'] != '' ) {
		$location = $_SERVER['HTTP_REFERER'];
		header ('Location: ' . $location);
	}
}


function jupc_posts_where($text) {
	global $wpdb, $wp_query;
	global $jupc_excluded_cat;

	jupc_load_excat();

	if (jupc_on() && jupc_excat_set()) {
		if (is_home() || is_archive())
			if (version_compare(get_bloginfo('version'), '2.3-RC1', '<')) {
				$text .= " AND ( '".$jupc_excluded_cat."' <> ALL (SELECT category_id FROM $wpdb->post2cat WHERE $wpdb->post2cat.post_id = $wpdb->posts.ID) )";
			}
			else {
				$text .= " AND ( '".$jupc_excluded_cat."' <> ALL (SELECT term_taxonomy_id FROM $wpdb->term_relationships WHERE $wpdb->term_relationships.object_id = $wpdb->posts.ID) )";
			}
	}
	
	return $text;
}

function jupc_load_excat() {
   global $jupc_excluded_cat;
   if( empty($jupc_excluded_cat) ) {
      $jupc_excluded_cat = get_option("jupc_excluded_cat");
      if( empty($jupc_excluded_cat) ) {
		 $jupc_excluded_cat = '-1';
         update_option("jupc_excluded_cat", $jupc_excluded_cat, 'ID of category to be filtered out when Parental Control is in effect.');
      }
   }
}

function jupc_load() {
	global $jupc_profanities;
	global $jupc_excluded_cat;
	jupc_load_excat();
	jupc_load_profanities();
}
   
function jupc_load_profanities() {
   global $jupc_profanities;
   if( empty($jupc_profanities) ) {
      $jupc_profanities = get_option("jupc_profanities");
      if( empty($jupc_profanities) ) {
        $jupc_profanities = array();
        update_option("jupc_profanities", $jupc_profanities, 'List of profanities to be filtered.' );
      }
   }
}

function jupc_make_config_page() {
	if ( function_exists('add_submenu_page') )
		add_submenu_page('plugins.php', 'Parental Control Configuration', 'Parental Control Configuration', 1, basename(__FILE__), 'jupc_config_page');
}

function jupc_config_page() {
   global $jupc_profanities;
   global $jupc_excluded_cat;
   jupc_load();

   if (isset($_POST['jupc_update'])) {
      ?><div class="updated"><p><strong>Parental Control configuration updated.</strong></p><?php
	  $jupc_excluded_cat = $_POST['jupc_excluded_cat'];
      update_option("jupc_excluded_cat", $jupc_excluded_cat );

      $prof_count = count($_POST['jupc_profanities']);                       
      sort( array_map('strtolower', $_POST['jupc_profanities'] ), SORT_STRING );
      $pstored = 0;
	  $prejected = 0;
      $jupc_profanities = array();
      $jupc_profanities_rejects = array();
      for( $p = 0; $p < $prof_count; $p++ ) {
         if( $_POST['jupc_profanities'][$p] > "" ) {
		    if (strlen($_POST['jupc_profanities'][$p]) > 1) {
				$jupc_profanities[$pstored] = $_POST['jupc_profanities'][$p];
				$pstored++;
			} else {
				$jupc_profanities_rejects[$prejected] = $_POST['jupc_profanities'][$p];
				$prejected++;
			}
         }
      }
      update_option("jupc_profanities", $jupc_profanities );
	  if (jupc_excat_set()) {
	    $excatname = get_cat_name($jupc_excluded_cat);
		print( "<p>Parental Controlled category set to <b>$jupc_excluded_cat</b> (\"$excatname\"). <br />" );
	  }
	  else
	    print( "<p>Parental Controlled category unset. <br />" );
	
	  if ($prejected > 0) {
	    if ($prejected > 1) {
	      echo "<b>$prejected</b> profanities rejected; minimum length is two letters. <br />";
		  // echo "The rejected words were: <b>" . $jupc_profanities_rejects[0] . "</b>";
		  // for ($i = 1; $i < $prejected; $i++)
		  //   echo ", <b>" . $jupc_profanities_rejects[$i] . "</b>";
		  // echo " <br />";
		}
		else {
		  echo "One profanity rejected; minimum length is two letters. <br />";
		  // echo "The rejected word was <b>" . $jupc_profanities_rejects[0] . "</b>. <br />";
		}
	  }
		
	  if ($pstored > 0)
		print( "<b>$pstored</b> profanities saved." );
	  else
		print( "All profanities erased." );
		
      ?></div><?php
   }?>

   <div class="wrap">
      <h2>Parental Control Configuration</h2>
      <form method="post" action="<?php print $_SERVER['PHP_SELF'] . '?page=' . basename(__FILE__); ?>">

		<fieldset id="Excategory"><legend><h3>Parental Control Category</h3></legend>
		  <p>Select the category you wish to exclude when Parental Control is in effect. Select empty for none.</p>
		  <p><?php
				if (jupc_excat_set())
					$curcat = $jupc_excluded_cat;
				else
					$curcat = -1;
					
				wp_dropdown_categories('hide_empty=0&name=jupc_excluded_cat&orderby=name&selected=' . $curcat .
				'&hierarchical=1&show_option_none= ');
		  ?></p>
		</fieldset>
	  
		<fieldset id="Profanities"><legend><h3>Profanities</h3></legend>
		  <p>Below you can add, remove or change words to be replaced by asterisks in posts and comments.
		  The amount of words you can add at once is limited; saving the current set of words will give
		  you new slots. Erasing a word before saving will remove it.</p>
          <table>
          <?php
            $pcount = 0;
            foreach($jupc_profanities as $tooltip) {
               $pcount++; 
            ?>               
            <tr>
               <td>
                  <input name="jupc_profanities[]" size="20" type="text" value="<?php print($tooltip);?>"/>
               </td>
            </tr>
            <?php
			}
            for($i=0; $i < 3; $i++) { $pcount++;
            ?>               
            <tr>
               <td>
                  <input name="jupc_profanities[]" size="20" type="text" value="" />
               </td>
            </tr>
            <?php } ?>
          </table>
		</fieldset>
        <br />
		<p><input type="submit" name="jupc_update" value="Save" /></p>
      </form>
   </div>
   <?php
}


// add the config menu
add_action('admin_menu', 'jupc_make_config_page');

add_filter('comment_text','jupc_profanity_filter');
add_filter('comment_author','jupc_profanity_filter');
add_filter('the_title','jupc_profanity_filter');
add_filter('the_content','jupc_profanity_filter');
add_filter('the_content','jupc_exclusion');
add_filter('the_excerpt','jupc_profanity_filter');
add_filter('the_excerpt','jupc_exclusion');

add_filter('posts_where', 'jupc_posts_where');

?>